@@ -1,7 +1,7 @@
 /*----------------------------------------------------------------------------
  ADOL-C -- Automatic Differentiation by Overloading in C++
  File:     detexam-old.cpp
- Revision: $Id: detexam-old.cpp 171 2010-10-04 13:57:19Z kulshres $
+ Revision: $Id: detexam-old.cpp 299 2012-03-21 16:08:40Z kulshres $
  Contents: computation of determinants
 
  Copyright (c) Andrea Walther, Andreas Griewank, Andreas Kowarz, 
@@ -130,7 +130,7 @@ int main() {
     fprintf(stdout,"\n %f =? %f should be the same \n",detout,diag);
 
     /*--------------------------------------------------------------------------*/
-    int tape_stats[STAT_SIZE];
+    size_t tape_stats[STAT_SIZE];
     tapestats(tag,tape_stats);
 
     fprintf(stdout,"\n    independents            %d\n",tape_stats[NUM_INDEPENDENTS]);
