@@ -1,11 +1,12 @@
 /*----------------------------------------------------------------------------
  ADOL-C -- Automatic Differentiation by Overloading in C++
  File:     oplate.h
- Revision: $Id: oplate.h 91 2010-02-24 07:56:58Z awalther $
+ Revision: $Id: oplate.h 360 2012-10-19 09:22:43Z kulshres $
  Contents: Numeric values for the various opcodes used by ADOL-C.
  
  Copyright (c) Andrea Walther, Andreas Griewank, Andreas Kowarz, 
-               Hristo Mitev, Sebastian Schlenkrich, Jean Utke, Olaf Vogel
+               Hristo Mitev, Sebastian Schlenkrich, Jean Utke, Olaf Vogel,
+               Kshitij Kulshreshtha
   
  This file is part of ADOL-C. This software is provided as open source.
  Any use, reproduction, or distribution of the software constitutes 
@@ -16,8 +17,6 @@
 #if !defined(ADOLC_OPLATE_P_H)
 #define ADOLC_OPLATE_P_H 1
 
-#include <common.h>
-
 /****************************************************************************/
 /* opcodes */
 
@@ -82,7 +81,31 @@ enum OPCODES {
   ceil_op,
   floor_op,
   ext_diff,
-  ignore_me
+  ignore_me,
+  subscript = 80,
+  subscript_ref,
+  ref_assign_d_zero,
+  ref_assign_d_one,
+  ref_assign_d,
+  ref_assign_a,
+  ref_assign_ind,
+  ref_incr_a,
+  ref_decr_a,
+  ref_eq_plus_d,
+  ref_eq_min_d,
+  ref_eq_plus_a,
+  ref_eq_min_a,
+  ref_eq_mult_d,
+  ref_eq_mult_a,
+  ref_copyout,
+  ref_cond_assign,
+  ref_cond_assign_s,
+  eq_a_a = 125,
+  neq_a_a,
+  le_a_a,
+  gt_a_a,
+  ge_a_a,
+  lt_a_a
 };
 
 /****************************************************************************/
