@@ -1,7 +1,7 @@
 /*----------------------------------------------------------------------------
  ADOL-C -- Automatic Differentiation by Overloading in C++
  File:     sparse/sparsedrivers.cpp
- Revision: $Id: sparsedrivers.cpp 296 2012-03-21 15:16:59Z kulshres $
+ Revision: $Id: sparsedrivers.cpp 373 2012-12-07 12:24:52Z kulshres $
  Contents: "Easy To Use" C++ interfaces of SPARSE package
  
  Copyright (c) Andrea Walther
@@ -12,7 +12,7 @@
   
 ----------------------------------------------------------------------------*/
 #include <adolc/sparse/sparsedrivers.h>
-#include <adolc/oplate.h>
+#include "oplate.h"
 #include <adolc/adalloc.h>
 #include <adolc/interfaces.h>
 #include "taping_p.h"
@@ -116,15 +116,15 @@ void generate_seed_jac
 {
   int dummy, i, j;
 
-  BipartiteGraphPartialColoringInterface *g = new BipartiteGraphPartialColoringInterface(SRC_MEM_ADOLC, JP, m, n);
+    BipartiteGraphPartialColoringInterface *g = new BipartiteGraphPartialColoringInterface(SRC_MEM_ADOLC, JP, m, n);
 
-  if (option == 1) 
-    g->GenerateSeedJacobian_unmanaged(Seed, p, &dummy, 
-				"SMALLEST_LAST","ROW_PARTIAL_DISTANCE_TWO"); 
-  else 
-    g->GenerateSeedJacobian_unmanaged(Seed, &dummy, p, 
-				"SMALLEST_LAST","COLUMN_PARTIAL_DISTANCE_TWO"); 
-  delete g;
+    if (option == 1) 
+      g->GenerateSeedJacobian_unmanaged(Seed, p, &dummy, 
+					"SMALLEST_LAST","ROW_PARTIAL_DISTANCE_TWO"); 
+    else 
+      g->GenerateSeedJacobian_unmanaged(Seed, &dummy, p, 
+					"SMALLEST_LAST","COLUMN_PARTIAL_DISTANCE_TWO"); 
+    delete g;
 
 }
 #else
@@ -152,7 +152,9 @@ int hess_pat(
     /* control option
        option : test the computational graph control flow
                                0 - safe mode (default)
-                               1 - tight mode                              */
+                               1 - tight mode
+                               2 - old safe mode 
+                               3 - old tight mode                         */
 
 ) {
     int         rc= -1;
@@ -166,10 +168,14 @@ int hess_pat(
         for (i=0; i<indep; i++)
             crs[i] = NULL;
 
-    if (( option < 0 ) || (option > 2 ))
+    if (( option < 0 ) || (option > 3 ))
       option = 0;   /* default */
 
-    if (option == 1)
+    if (option == 3)
+	rc = nonl_ind_old_forward_tight(tag, 1, indep, basepoint, crs);
+    else if (option == 2)
+	rc = nonl_ind_old_forward_safe(tag, 1, indep, basepoint, crs);
+    else if (option == 1)
       rc = nonl_ind_forward_tight(tag, 1, indep, basepoint, crs);
     else
       rc = nonl_ind_forward_safe(tag, 1, indep, basepoint, crs);
@@ -429,6 +435,8 @@ int sparse_hess(
                     options[0] :test the computational graph control flow
                                0 - safe mode (default)
                                1 - tight mode
+			       2 - old safe mode
+			       3 - old tight mode
                     options[1] : way of recovery
                                0 - indirect recovery
                                1 - direct recovery                         */
@@ -452,7 +460,7 @@ int sparse_hess(
 
     /* Generate sparsity pattern, determine nnz, allocate memory */
     if (repeat <= 0) {
-        if (( options[0] < 0 ) || (options[0] > 1 ))
+        if (( options[0] < 0 ) || (options[0] > 3 ))
           options[0] = 0; /* default */
         if (( options[1] < 0 ) || (options[1] > 1 ))
           options[1] = 0; /* default */
